'use strict';

import Constants from './constants';

/**
 *
 */
export default class ActiveSpan {

    /**
     * [constructor description]
     * @return {[type]} [description]
     */
    constructor() {
        this._imp = null;
    }

    // ---------------------------------------------------------------------- //
    // Span properties
    // ---------------------------------------------------------------------- //

    /**
     * [uuid description]
     * @return {[type]} [description]
     */
    uuid() {
        if (arguments.length !== 0) {
            throw Error('Invalid arguments');
        }

        if (!this._imp) {
            return '';
        }
        return this._imp.uuid();
    }

    /**
     * [operation description]
     * @return {[type]} [description]
     */
    operation(name) {
        if (arguments.length > 1) {
            throw new Error('Invalid arguments');
        }

        if (arguments.length === 0) {
            if (!this._imp) {
                return '';
            } else {
                return this._imp.getOperation(...arguments);
            }
        } else {
            if (typeof name !== 'string') {
                throw new Error('Invalid argument type');
            }

            if (!this._imp) {
                return;
            } else {
                this._imp.setOperation(...arguments);
            }
        }
    }

    /**
     * [tags description]
     * @return {[type]} [description]
     */
    tags(map) {
        if (arguments.length > 1) {
            throw new Error('Invalid arguments');
        }

        if (arguments.length === 0) {
            if (!this._imp) {
                return {};
            } else {
                return this._imp.getTags(...arguments);
            }
        } else {
            if (typeof map !== 'object') {
                throw new Error('Invalid argument type');
            }

            if (!this._imp) {
                return;
            } else {
                this._imp.setTags(...arguments);
            }
        }
    }

    // ---------------------------------------------------------------------- //
    // Span lifetime
    // ---------------------------------------------------------------------- //

    /**
     * [end description]
     * @return {[type]} [description]
     */
    end() {
        if (arguments.length !== 0) {
            throw new Error('Invalid arguments');
        }

        if (!this._imp) {
            return;
        }
        this._imp.end(...arguments);
    }

    // ---------------------------------------------------------------------- //
    // Span heirarchy
    // ---------------------------------------------------------------------- //

    /**
     * Sets the parent span explicitly.  If the argument is a string, then it
     * will be interpreted as the parent span's UUID. Otherwise, if the argument
     * is an object, then it is expected to be a valid ActiveSpan object.
     *
     * Note that a getter interface is not supported here as an arbitrary span
     * cannot reliably return its parent object in all cases.
     *
     * @param  {[type]} span [description]
     * @return {[type]}      [description]
     */
    parent(span) {
        if (arguments.length !== 1) {
            throw new Error('Invalid arguments');
        }
        let spanType = typeof span;
        if (spanType !== 'string' && spanType !== 'object') {
            throw new Error('Expected a string or ActiveSpan object');
        }

        if (!this._imp) {
            return;
        }
        this._imp.parent(...arguments);
    }

    /**
     * [child description]
     * @param  {[type]} operationName [description]
     * @return {[type]}               [description]
     */
    child(operationName) {
        if (arguments.length !== 1) {
            throw new Error('Invalid arguments');
        }
        if (typeof operationName !== 'string') {
            throw new Error('Expected operation name to be a string');
        }

        if (!this._imp) {
            return new ActiveSpan(null);
        }
        return this._imp.child(...arguments);
    }

    // ---------------------------------------------------------------------- //
    // Log records
    // ---------------------------------------------------------------------- //

    /**
     * Creates a new log record with a manually specified set of fields.
     *
     * The underlying implementation is expected to automatically set the
     * `timestamp_micros` field if it is not specified explicitly.
     *
     * @param  {[type]} fields [description]
     */
    log(fields) {
        if (arguments.length !== 1) {
            throw new Error('Invalid arguments');
        }
        if (fields === null) {
            return;
        }
        if (typeof fields !== 'object') {
            throw new Error('Invalid arguments');
        }

        if (!this._imp) {
            return;
        }
        this._imp.log(...arguments);
    }

    /**
     * Creates an information log record with the given message and optional
     * payload.
     *
     * @param  {string} message Text of the log record
     * @param  {object} payload [description]
     * @return {[type]}         [description]
     */
    info(message, payload) {
        if (arguments.length < 1 || arguments.length > 2) {
            throw new Error('Invalid arguments.');
        }
        if (typeof message !== 'string') {
            throw new Error('Expected message to be a string');
        }

        if (!this._imp) {
            return;
        }
        this._imp.log({
            severity : Constants.LOG_SEVERITY_INFO,
            message  : message,
            payload  : payload,
        });
    }

    /**
     * Creates a warning log record with the given message and optional payload.
     *
     * @param  {[type]} message [description]
     * @param  {[type]} payload [description]
     * @return {[type]}         [description]
     */
    warn(message, payload) {
        if (arguments.length < 1 || arguments.length > 2) {
            throw new Error('Invalid arguments.');
        }
        if (typeof message !== 'string') {
            throw new Error('Expected message to be a string');
        }

        if (!this._imp) {
            return;
        }
        this._imp.log({
            severity : Constants.LOG_SEVERITY_WARN,
            message  : message,
            payload  : payload,
        });
    }

    /**
     * [error description]
     * @param  {[type]} message [description]
     * @param  {[type]} payload [description]
     * @return {[type]}         [description]
     */
    error(message, payload) {
        if (arguments.length < 1 || arguments.length > 2) {
            throw new Error('Invalid arguments.');
        }
        if (typeof message !== 'string') {
            throw new Error('Expected message to be a string');
        }

        if (!this._imp) {
            return;
        }
        this._imp.log({
            severity : Constants.LOG_SEVERITY_ERROR,
            message  : message,
            payload  : payload,
        });
    }

    /**
     * [fatal description]
     * @param  {[type]} message [description]
     * @param  {[type]} payload [description]
     * @return {[type]}         [description]
     */
    fatal(message, payload) {
        if (arguments.length < 1 || arguments.length > 2) {
            throw new Error('Invalid arguments.');
        }
        if (typeof message !== 'string') {
            throw new Error('Expected message to be a string');
        }

        if (!this._imp) {
            return;
        }
        this._imp.log({
            severity : Constants.LOG_SEVERITY_FATAL,
            message  : message,
            payload  : payload,
        });
    }
}
