// For the convenience of unit testing, add these to the global namespace
global._      = require('underscore');
global.expect = require('chai').expect;
global.Tracer = require('../dist/opentracing-node.js');

describe('OpenTracing', function() {

    describe('Default no-op implementation', function() {

        it('should have the required functions', function() {
            expect(Tracer.uuid).to.be.a('function');

            expect(Tracer.on).to.be.a('function');
            expect(Tracer.once).to.be.a('function');
            expect(Tracer.removeListener).to.be.a('function');
            expect(Tracer.emit).not.to.be.a('function');

            expect(Tracer.initialize).to.be.a('function');
            expect(Tracer.options).to.be.a('function');
            expect(Tracer.span).to.be.a('function');
            expect(Tracer.flush).to.be.a('function');
        });

        it('should respect Tracer API arguments constraints', function() {
            expect(Tracer.uuid()).to.be.a('string');
            expect(function() { Tracer.uuid('123456789'); }).to.throw();
            expect(function() { Tracer.uuid(123456789); }).to.throw();

            expect(function() { Tracer.initialize(); }).to.throw();
            expect(function() { Tracer.initialize({}); }).not.to.throw();
            expect(function() { Tracer.initialize({}, true); }).to.throw();

            expect(Tracer.options()).to.be.an('object');
            expect(function() { Tracer.options({}); }).not.to.throw();
            expect(function() { Tracer.options({}, true); }).to.throw();

            expect(Tracer.span("test")).to.be.an('object');
            expect(function() { Tracer.span(); }).to.throw();
            expect(function() { Tracer.span(null); }).to.throw();
            expect(function() { Tracer.span(123); }).to.throw();

            expect(function() { Tracer.flush(); }).not.to.throw();
            expect(function() { Tracer.flush(true); }).to.throw();
            expect(function() { Tracer.flush(function(){}); }).not.to.throw();
            expect(function() { Tracer.flush(function(){}, true); }).to.throw();
        });

        it('should respect ActiveSpan API argument constraints', function() {
            var span = Tracer.span('test');

            expect(span.uuid()).to.be.a('string');
            expect(function() { span.uuid('123456789'); }).to.throw();
            expect(function() { span.uuid(123456789); }).to.throw();

            expect(span.operation()).to.be.a('string');
            expect(function(){ span.operation('new_name'); }).not.to.throw();
            expect(function(){ span.operation(123); }).to.throw();
            expect(function(){ span.operation('new_name', 123); }).to.throw();

            expect(span.tags()).to.be.a('object');
            expect(span.tags({})).to.be.an('undefined');
            expect(function(){ span.tags({}); }).not.to.throw();
            expect(function(){ span.tags({ 'key' : 'value' }); }).not.to.throw();
            expect(function(){ span.tags('key'); }).to.throw();
            expect(function(){ span.tags(123); }).to.throw();
            expect(function(){ span.tags('tag', 'value'); }).to.throw();

            expect(function(){ span.parent('123'); }).not.to.throw();
            expect(function(){ span.parent(); }).to.throw();
            expect(function(){ span.parent(123); }).to.throw();
            expect(function(){ span.parent(true); }).to.throw();
            expect(function(){ span.parent('test', true); }).to.throw();

            expect(span.child('123')).to.be.an('object');
            expect(function(){ span.child(); }).to.throw();
            expect(function(){ span.child(null); }).to.throw();
            expect(function(){ span.child(123); }).to.throw();
            expect(function(){ span.child({}); }).to.throw();

            expect(span.log({})).to.be.an('undefined');
            expect(function(){ span.log(); }).to.throw();
            expect(function(){ span.log('message') }).to.throw();
            expect(function(){ span.log({}); }).not.to.throw();
            expect(function(){ span.log({}, true) }).to.throw();

            expect(span.info('message')).to.be.an('undefined');
            expect(function(){ span.info(); }).to.throw();
            expect(function(){ span.info('message') }).not.to.throw();
            expect(function(){ span.info('message', { key : 'value' }); }).not.to.throw();
            expect(function(){ span.info('message', true) }).not.to.throw();

            expect(span.warn('message')).to.be.an('undefined');
            expect(function(){ span.warn(); }).to.throw();
            expect(function(){ span.warn('message') }).not.to.throw();
            expect(function(){ span.warn('message', { key : 'value' }); }).not.to.throw();
            expect(function(){ span.warn('message', true) }).not.to.throw();

            expect(span.error('message')).to.be.an('undefined');
            expect(function(){ span.error(); }).to.throw();
            expect(function(){ span.error('message') }).not.to.throw();
            expect(function(){ span.error('message', { key : 'value' }); }).not.to.throw();
            expect(function(){ span.error('message', true) }).not.to.throw();

            // No tests for fatal beyond existence of the function
            expect(span.fatal).to.be.a('function');

            expect(function(){ span.end(); }).not.to.throw();
            expect(function(){ span.end(true); }).to.throw();
        });
    });
});
